==> grib-api_1.9.0.orig.tar.gz: grib_api-1.9.0/src/grib_accessor_class_decimal_precision.c <==
/**
* Copyright 2005-2007 ECMWF
* 
* Licensed under the GNU Lesser General Public License which
* incorporates the terms and conditions of version 3 of the GNU
* General Public License.
* See LICENSE and gpl-3.0.txt for details.
*/

/**********************************
 *  Enrico Fucile
 *********************************/

#include "grib_api_internal.h"
/*
   This is used by make_class.pl

   START_CLASS_DEF
   CLASS      = accessor
   SUPER      = grib_accessor_class_long
   IMPLEMENTS = unpack_long; pack_long
   IMPLEMENTS = init
   MEMBERS = const char* values
   MEMBERS = const char* bits_per_value
   MEMBERS = const char* changing_precision
   MEMBERS = const char* decimal_scale_factor
   END_CLASS_DEF
 */

/* START_CLASS_IMP */

/*

Don't edit anything between START_CLASS_IMP and END_CLASS_IMP
Instead edit values between START_CLASS_DEF and END_CLASS_DEF
or edit "accessor.class" and rerun ./make_class.pl

*/

static int pack_long(grib_accessor*, const long* val,size_t *len);
static int unpack_long(grib_accessor*, long* val,size_t *len);
static void init(grib_accessor*,const long, grib_arguments* );
static void init_class(grib_accessor_class*);

typedef struct grib_accessor_decimal_precision {
    grib_accessor          att;
/* Members defined in gen */
/* Members defined in long */
/* Members defined in decimal_precision */
	const char* values;
	const char* bits_per_value;
	const char* changing_precision;
	const char* decimal_scale_factor;
} grib_accessor_decimal_precision;

extern grib_accessor_class* grib_accessor_class_long;

static grib_accessor_class _grib_accessor_class_decimal_precision = {
    &grib_accessor_class_long,                      /* super                     */
    "decimal_precision",                      /* name                      */
    sizeof(grib_accessor_decimal_precision),  /* size                      */
    0,                           /* inited */
    &init_class,                 /* init_class */
    &init,                       /* init                      */
    0,                  /* post_init                      */
    0,                    /* free mem                       */
    0,                       /* describes himself         */
    0,                /* get length of section     */
    0,                /* get number of values      */
    0,                 /* get number of bytes      */
    0,                /* get offset to bytes           */
    0,            /* get native type               */
    0,                /* get sub_section                */
    0,               /* grib_pack procedures long      */
    0,               /* grib_pack procedures long      */
    &pack_long,                  /* grib_pack procedures long      */
    &unpack_long,                /* grib_unpack procedures long    */
    0,                /* grib_pack procedures double    */
    0,              /* grib_unpack procedures double  */
    0,                /* grib_pack procedures string    */
    0,              /* grib_unpack procedures string  */
    0,                 /* grib_pack procedures bytes     */
    0,               /* grib_unpack procedures bytes   */
    0,            /* pack_expression */
    0,              /* notify_change   */
    0,                /* update_size   */
    0,            /* preferred_size   */
    0,                    /* resize   */
    0,      /* nearest_smaller_value */
    0,                       /* next accessor    */
    0,                    /* compare vs. another accessor   */
    0,     /* unpack only ith value          */
    0,             		/* clear          */
};


grib_accessor_class* grib_accessor_class_decimal_precision = &_grib_accessor_class_decimal_precision;


static void init_class(grib_accessor_class* c)
{
	c->dump	=	(*(c->super))->dump;
	c->next_offset	=	(*(c->super))->next_offset;
	c->value_count	=	(*(c->super))->value_count;
	c->byte_count	=	(*(c->super))->byte_count;
	c->byte_offset	=	(*(c->super))->byte_offset;
	c->get_native_type	=	(*(c->super))->get_native_type;
	c->sub_section	=	(*(c->super))->sub_section;
	c->pack_missing	=	(*(c->super))->pack_missing;
	c->is_missing	=	(*(c->super))->is_missing;
	c->pack_double	=	(*(c->super))->pack_double;
	c->unpack_double	=	(*(c->super))->unpack_double;
	c->pack_string	=	(*(c->super))->pack_string;
	c->unpack_string	=	(*(c->super))->unpack_string;
	c->pack_bytes	=	(*(c->super))->pack_bytes;
	c->unpack_bytes	=	(*(c->super))->unpack_bytes;
	c->pack_expression	=	(*(c->super))->pack_expression;
	c->notify_change	=	(*(c->super))->notify_change;
	c->update_size	=	(*(c->super))->update_size;
	c->preferred_size	=	(*(c->super))->preferred_size;
	c->resize	=	(*(c->super))->resize;
	c->nearest_smaller_value	=	(*(c->super))->nearest_smaller_value;
	c->next	=	(*(c->super))->next;
	c->compare	=	(*(c->super))->compare;
	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
	c->clear	=	(*(c->super))->clear;
}

/* END_CLASS_IMP */

static void init(grib_accessor* a,const long l, grib_arguments* args)
{
  int n=0;
  grib_accessor_decimal_precision* self= (grib_accessor_decimal_precision*)a;
  
  self->bits_per_value=grib_arguments_get_name(a->parent->h,args,n++);
  self->decimal_scale_factor=grib_arguments_get_name(a->parent->h,args,n++);
  self->changing_precision=grib_arguments_get_name(a->parent->h,args,n++);
  self->values=grib_arguments_get_name(a->parent->h,args,n++);
  
  a->flags |= GRIB_ACCESSOR_FLAG_FUNCTION;
  a->length=0;
}

static int  unpack_long(grib_accessor* a, long* val, size_t *len)
{
  int ret=0;
  grib_accessor_decimal_precision* self= (grib_accessor_decimal_precision*)a;
  grib_handle* h=a->parent->h;

  if((ret = grib_get_long_internal(h,self->decimal_scale_factor,val))
       != GRIB_SUCCESS) return ret;

  *len =1;
  return ret;
}

static int pack_long(grib_accessor* a, const long* val, size_t *len)
{
  long bitsPerValue=0;
  double* values=NULL;
  size_t size=0;
  int ret=0;
  grib_accessor_decimal_precision* self= (grib_accessor_decimal_precision*)a;
  grib_context* c=a->parent->h->context;
  grib_handle* h=a->parent->h;

  if (!self->values) {
    if((ret = grib_set_long_internal(h, self->bits_per_value,0))
        != GRIB_SUCCESS) return ret;
        
    if((ret = grib_set_long_internal(h, self->decimal_scale_factor,*val))
        != GRIB_SUCCESS) return ret;

    if((ret = grib_set_long_internal(h, self->changing_precision,1))
        != GRIB_SUCCESS) {
      grib_context_free(c,values);
      return ret;
        }

    return GRIB_SUCCESS;
  }

  if ( (ret=grib_get_size(h,self->values,&size)) != GRIB_SUCCESS) return ret;

  values=(double*)grib_context_malloc(c,size*sizeof(double));
  if (!values) return GRIB_OUT_OF_MEMORY;

  if((ret = grib_get_double_array_internal(h,self->values,values,&size))
       != GRIB_SUCCESS) {
        grib_context_buffer_free(c,values);
        return ret;
  }

  if((ret = grib_set_long_internal(h, self->decimal_scale_factor,*val))
      != GRIB_SUCCESS) { 
        grib_context_buffer_free(c,values);
	  return ret;
	  }

  if((ret = grib_set_long_internal(h, self->bits_per_value,bitsPerValue))
      != GRIB_SUCCESS) {
        grib_context_free(c,values);
	  return ret;
	 }

     if((ret = grib_set_long_internal(h, self->changing_precision,1))
         != GRIB_SUCCESS) {
       grib_context_free(c,values);
       return ret;
         }

  if((ret = grib_set_double_array_internal(h, self->values,values,size))
      != GRIB_SUCCESS) {
        grib_context_buffer_free(c,values);
	  return ret;
	 }

  grib_context_free(c,values);

  return GRIB_SUCCESS;
}

==> grib-api_1.10.4.orig.tar.gz: grib_api-1.10.4/src/grib_accessor_class_decimal_precision.c <==
/*
 * Copyright 2005-2013 ECMWF.
 *
 * This software is licensed under the terms of the Apache Licence Version 2.0
 * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
 * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
 */

/**********************************
 *  Enrico Fucile
 *********************************/

#include "grib_api_internal.h"
/*
   This is used by make_class.pl

   START_CLASS_DEF
   CLASS      = accessor
   SUPER      = grib_accessor_class_long
   IMPLEMENTS = unpack_long; pack_long
   IMPLEMENTS = init
   MEMBERS = const char* values
   MEMBERS = const char* bits_per_value
   MEMBERS = const char* changing_precision
   MEMBERS = const char* decimal_scale_factor
   END_CLASS_DEF
 */

/* START_CLASS_IMP */

/*

Don't edit anything between START_CLASS_IMP and END_CLASS_IMP
Instead edit values between START_CLASS_DEF and END_CLASS_DEF
or edit "accessor.class" and rerun ./make_class.pl

*/

static int pack_long(grib_accessor*, const long* val,size_t *len);
static int unpack_long(grib_accessor*, long* val,size_t *len);
static void init(grib_accessor*,const long, grib_arguments* );
static void init_class(grib_accessor_class*);

typedef struct grib_accessor_decimal_precision {
    grib_accessor          att;
/* Members defined in gen */
/* Members defined in long */
/* Members defined in decimal_precision */
	const char* values;
	const char* bits_per_value;
	const char* changing_precision;
	const char* decimal_scale_factor;
} grib_accessor_decimal_precision;

extern grib_accessor_class* grib_accessor_class_long;

static grib_accessor_class _grib_accessor_class_decimal_precision = {
    &grib_accessor_class_long,                      /* super                     */
    "decimal_precision",                      /* name                      */
    sizeof(grib_accessor_decimal_precision),  /* size                      */
    0,                           /* inited */
    &init_class,                 /* init_class */
    &init,                       /* init                      */
    0,                  /* post_init                      */
    0,                    /* free mem                       */
    0,                       /* describes himself         */
    0,                /* get length of section     */
    0,              /* get length of string      */
    0,                /* get number of values      */
    0,                 /* get number of bytes      */
    0,                /* get offset to bytes           */
    0,            /* get native type               */
    0,                /* get sub_section                */
    0,               /* grib_pack procedures long      */
    0,               /* grib_pack procedures long      */
    &pack_long,                  /* grib_pack procedures long      */
    &unpack_long,                /* grib_unpack procedures long    */
    0,                /* grib_pack procedures double    */
    0,              /* grib_unpack procedures double  */
    0,                /* grib_pack procedures string    */
    0,              /* grib_unpack procedures string  */
    0,                 /* grib_pack procedures bytes     */
    0,               /* grib_unpack procedures bytes   */
    0,            /* pack_expression */
    0,              /* notify_change   */
    0,                /* update_size   */
    0,            /* preferred_size   */
    0,                    /* resize   */
    0,      /* nearest_smaller_value */
    0,                       /* next accessor    */
    0,                    /* compare vs. another accessor   */
    0,     /* unpack only ith value          */
    0,     /* unpack a subarray         */
    0,             		/* clear          */
};


grib_accessor_class* grib_accessor_class_decimal_precision = &_grib_accessor_class_decimal_precision;


static void init_class(grib_accessor_class* c)
{
	c->dump	=	(*(c->super))->dump;
	c->next_offset	=	(*(c->super))->next_offset;
	c->string_length	=	(*(c->super))->string_length;
	c->value_count	=	(*(c->super))->value_count;
	c->byte_count	=	(*(c->super))->byte_count;
	c->byte_offset	=	(*(c->super))->byte_offset;
	c->get_native_type	=	(*(c->super))->get_native_type;
	c->sub_section	=	(*(c->super))->sub_section;
	c->pack_missing	=	(*(c->super))->pack_missing;
	c->is_missing	=	(*(c->super))->is_missing;
	c->pack_double	=	(*(c->super))->pack_double;
	c->unpack_double	=	(*(c->super))->unpack_double;
	c->pack_string	=	(*(c->super))->pack_string;
	c->unpack_string	=	(*(c->super))->unpack_string;
	c->pack_bytes	=	(*(c->super))->pack_bytes;
	c->unpack_bytes	=	(*(c->super))->unpack_bytes;
	c->pack_expression	=	(*(c->super))->pack_expression;
	c->notify_change	=	(*(c->super))->notify_change;
	c->update_size	=	(*(c->super))->update_size;
	c->preferred_size	=	(*(c->super))->preferred_size;
	c->resize	=	(*(c->super))->resize;
	c->nearest_smaller_value	=	(*(c->super))->nearest_smaller_value;
	c->next	=	(*(c->super))->next;
	c->compare	=	(*(c->super))->compare;
	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
	c->unpack_double_subarray	=	(*(c->super))->unpack_double_subarray;
	c->clear	=	(*(c->super))->clear;
}

/* END_CLASS_IMP */

static void init(grib_accessor* a,const long l, grib_arguments* args)
{
  int n=0;
  grib_accessor_decimal_precision* self= (grib_accessor_decimal_precision*)a;
  
  self->bits_per_value=grib_arguments_get_name(a->parent->h,args,n++);
  self->decimal_scale_factor=grib_arguments_get_name(a->parent->h,args,n++);
  self->changing_precision=grib_arguments_get_name(a->parent->h,args,n++);
  self->values=grib_arguments_get_name(a->parent->h,args,n++);
  
  a->flags |= GRIB_ACCESSOR_FLAG_FUNCTION;
  a->length=0;
}

static int  unpack_long(grib_accessor* a, long* val, size_t *len)
{
  int ret=0;
  grib_accessor_decimal_precision* self= (grib_accessor_decimal_precision*)a;
  grib_handle* h=a->parent->h;

  if((ret = grib_get_long_internal(h,self->decimal_scale_factor,val))
       != GRIB_SUCCESS) return ret;

  *len =1;
  return ret;
}

static int pack_long(grib_accessor* a, const long* val, size_t *len)
{
  long bitsPerValue=0;
  double* values=NULL;
  size_t size=0;
  int ret=0;
  grib_accessor_decimal_precision* self= (grib_accessor_decimal_precision*)a;
  grib_context* c=a->parent->h->context;
  grib_handle* h=a->parent->h;

  if (!self->values) {
    if((ret = grib_set_long_internal(h, self->bits_per_value,0))
        != GRIB_SUCCESS) return ret;
        
    if((ret = grib_set_long_internal(h, self->decimal_scale_factor,*val))
        != GRIB_SUCCESS) return ret;

    if((ret = grib_set_long_internal(h, self->changing_precision,1))
        != GRIB_SUCCESS) {
      grib_context_free(c,values);
      return ret;
        }

    return GRIB_SUCCESS;
  }

  if ( (ret=grib_get_size(h,self->values,&size)) != GRIB_SUCCESS) return ret;

  values=(double*)grib_context_malloc(c,size*sizeof(double));
  if (!values) return GRIB_OUT_OF_MEMORY;

  if((ret = grib_get_double_array_internal(h,self->values,values,&size))
       != GRIB_SUCCESS) {
        grib_context_buffer_free(c,values);
        return ret;
  }

  if((ret = grib_set_long_internal(h, self->decimal_scale_factor,*val))
      != GRIB_SUCCESS) { 
        grib_context_buffer_free(c,values);
	  return ret;
	  }

  if((ret = grib_set_long_internal(h, self->bits_per_value,bitsPerValue))
      != GRIB_SUCCESS) {
        grib_context_free(c,values);
	  return ret;
	 }

     if((ret = grib_set_long_internal(h, self->changing_precision,1))
         != GRIB_SUCCESS) {
       grib_context_free(c,values);
       return ret;
         }

  if((ret = grib_set_double_array_internal(h, self->values,values,size))
      != GRIB_SUCCESS) {
        grib_context_buffer_free(c,values);
	  return ret;
	 }

  grib_context_free(c,values);

  return GRIB_SUCCESS;
}
